importPackage(java.lang);
importPackage(java.util);
importPackage(java.io);
importPackage(java.text);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessageImpl);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);

var xStream = new XStream(new PureJavaReflectionProvider())

load("integrationservices/hpnnmi/nnmi-config.js");
load("integrationservices/hpnnmi/baseclass.js");
load("integrationservices/hpnnmi/logger.js");
load("integrationservices/hpnnmi/wsutil.js");
load("integrationservices/hpnnmi/nnmi-lifecycle.js");
load("integrationservices/hpnnmi/nnmi-subscription.js");
load("integrationservices/hpnnmi/nnmi-subscription-thread.js");
load("integrationservices/hpnnmi/nnmi-response.js");
load("integrationservices/hpnnmi/hpnnmi-mg.js");

var log = new Logger();
var preamble = "[hpnnmi.js] ";

var nnmiResponse = new NNMIResponse(NNMI_URL, NNMI_USER, NNMI_PASSWORD);

/**
 * This is the injection point to xMatters from NNMi Subscription
 *  
 * This method will parse the NNMi callback request and generate an APXML Message to be 
 * sent to xMatters via the ServiceAPI.
 *
 * Modify the apxml object to add tokens injected to xMatters.
 * NNMi incident information can be retrieved from the E4X XML object 'content'
 *
 * For example use the following command to retrieve the uuid:
 *   content.uuid.text();
 */
function apia_http( httpRequestProperties )
{
  // create apxml from http request 
  log.debug(preamble+"Incoming HTTP Request: " + httpRequestProperties);


  var requestBody = httpRequestProperties.getProperty("REQUEST_BODY");
  if ( requestBody == null )
  {
    log.debug(preamble+"Received unrecognize Request, no notification was created. (Suppressed)");
    return null;
  }

  var content = new XML(requestBody)..arg0;
  // build apxml content
  var apxml = new XML("<transaction>" +
                      "  <header><method>Add</method><subclass>Action</subclass></header>" +
                      "  <data></data>" +
                      "</transaction>");
  var data = apxml.data;
  var incidentName = content.name.text();
  if ( incidentName.length() <= 0)
  {
    log.debug(preamble+"Received unrecognize Request, no notification was created. (Suppressed)");
    return "<error>No xMatters Event was generated as the required node <arg0><name/></arg0> was not found in the request body.</error>";
  }
  data.appendChild(new XML("<action_script_set>" + ACTION_SCRIPT + "</action_script_set>"));
  // determine recipient via name of incident
  if ("AddressNotResponding".equals(incidentName))
  {
    data.appendChild(<recipients>Operations</recipients>);
  }
  else if ("ConnectionDown".equals(incidentName))
  {
    data.appendChild(<recipients>Operations</recipients>);
  }
  else if ("InterfaceDown".equals(incidentName))
  {
    data.appendChild(<recipients>Operations</recipients>);
  }
  else if ("NodeDown".equals(incidentName))
  {
    data.appendChild(<recipients>Operations</recipients>);
  }
  else
  {
    data.appendChild(<recipients>Operations</recipients>);
  }
  data.appendChild(<fyi>no</fyi>);
  // direct copy of incident detail from NNMi request
  data.appendChild(<incident_id>{content.uuid.text()}</incident_id>);
  data.appendChild(<id>{content.id.text()}</id>);
  data.appendChild(<uuid>{content.uuid.text()}</uuid>);
  data.appendChild(<name>{content.name.text()}</name>);
  data.appendChild(<sourceUuid>{content.sourceUuid.text()}</sourceUuid>);
  data.appendChild(<sourceObjectName>{content.sourceName.text()}</sourceObjectName>);
  data.appendChild(<sourceNodeUuid>{content.sourceNodeUuid.text()}</sourceNodeUuid>);
  data.appendChild(<sourceNodeName>{content.sourceNodeName.text()}</sourceNodeName>);
  data.appendChild(<sourceNodeLongName>{content.sourceNodeLongName.text()}</sourceNodeLongName>);
  data.appendChild(<lifecycleState>{content.lifecycleState.text()}</lifecycleState>);
  data.appendChild(<severity>{content.severity.text()}</severity>);
  data.appendChild(<priority>{content.priority.text()}</priority>);
  data.appendChild(<category>{content.category.text()}</category>);
  data.appendChild(<family>{content.family.text()}</family>);
  data.appendChild(<nature>{content.nature.text()}</nature>);
  data.appendChild(<origin>{content.origin.text()}</origin>);
  data.appendChild(<duplicateCount>{content.duplicateCount.text()}</duplicateCount>);
  data.appendChild(<rcaActive>{content.rcaActive.text()}</rcaActive>);
  data.appendChild(<formattedMessage>{content.formattedMessage.text()}</formattedMessage>);
  data.appendChild(<originOccurrenceTime>{content.originOccurrenceTime.text()}</originOccurrenceTime>);
  data.appendChild(<firstOccurrenceTime>{content.firstOccurrenceTime.text()}</firstOccurrenceTime>);
  data.appendChild(<lastOccurrenceTime>{content.lastOccurrenceTime.text()}</lastOccurrenceTime>);
  data.appendChild(<incidentResent>{content.incidentResent.text()}</incidentResent>);
  data.appendChild(<created>{content.created.text()}</created>);
  data.appendChild(<updateTime>{content.updateTime.text()}</updateTime>);
  data.appendChild(<previousLifecycleState>{content.previousLifecycleState.text()}</previousLifecycleState>);
  data.appendChild(<previousRcaActive>{content.previousRcaActive.text()}</previousRcaActive>);

  // include cias
  var cias = content.cias;
  for (var i=0; i < cias.length(); i++)
  {
    var ciasName = cias[i].name.text();
    var ciasValue = cias[i].value.text();
    data.appendChild(new XML("<"+ciasName+">"+ciasValue+"</"+ciasName+">"));
  }

  // add custom tokens
  // data.appendChild(<custom_parameter>{content.incidentResent.text()}</custom_parameter>);
  // data.appendChild(<custom2>static custom token value</custom2>);

  ServiceAPI.sendAPXML(new APXMLMessageImpl(apxml.toString()));

  return apxml.toString();
}

/**
 * This is the main method for handling APXML messages sent to the 
 * Integration Service by APClient.  The messages may be requests to perform 
 * local activity, or they may be requests to make submissions to AlarmPoint.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml)
{
  // Enrich by adding/updating the timestamp token with the current time.
  // 
  // NOTE: the Date must be converted to a string; otherwise, it gets wrapped
  // in a JavaScript envelope whose toString() method returns an
  // object identifier.
  apxml.setToken("timestamp", (new Date()).toString());  

  // Forward enriched APXML to AlarmPoint.
  return apxml;  
}

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by AlarmPoint.  The messages may be responses to 
 * previous submissions, or they may be requests to the Integration Service
 * that originate from AlarmPoint 
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * either via the Integration Service's outbound queues or directly as a 
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml) 
{
  var method = apxml.getMethod();
  
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  } 
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  } 
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}

/**
 * OK responses are returned by AlarmPoint for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * Log the acknowledgement using the nnmi Integration Service's logger.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  log.info("Submission of transaction " + id + "succeeded.");
}

/**
 * ERROR responses are returned by AlarmPoint for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * Log the error using the nnmi Integration Service's logger.
 */
function handleERROR(apxml) 
{
  var id = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE); 
  log.error("Submission of transaction " + id + "failed with error code " + code);
}

/**
 * Send is unused by the nnmi Integration Service.
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml) {
  var requestText = apxml.getValue("request_text");
  var messageText = apxml.getValue("message_text");
  var incidentId = apxml.getValue("incident_id");
  var nnmid; 
  var newPriority;
  
  if (!incidentId)
  {
     var errorMessage = "Request Script Failed: Request not associated with an Incident";
     log.error(errorMessage);
     return errorMessage;  
  }
  
  log.debug("incidentId: " + incidentId);
  log.debug("requestText: " + requestText);
  log.debug("messageText: " + messageText);
  
  // deserialize the additional tokens
  if (apxml.getValue("additionalTokens") != null && apxml.getValue("additionalTokens") != "") 
  {
    var additionalTokens = xStream.fromXML(apxml.getValue("additionalTokens")); // LinkedHashMap
    
    log.debug( "additionalTokens: " + additionalTokens);
    nnmid = additionalTokens.get("nnmid"); 
  }
  else
  {
    var errorMessage = "additionalTokens is missing from apxml";
    log.debug(errorMessage);
    return errorMessage; 
  }
  
  log.info("Processing " + requestText + " type response");
  
  // Handles both annotate and ignore.
  if (requestText == "annotate")
  {
      log.debug(requestText);
	  if( !nnmid )
	  {
	    nnmid = nnmiResponse.getNNMiIncidentId( incidentId );
	  }
	  log.debug("NNMID" + nnmid);
	  nnmiResponse.annotate(nnmid, messageText);
  }
  else
  {
      var errorMessage = "Unexpected request_text value: " + requestText; 
      log.error(errorMessage);
  }
}
 
/**
 * Only nnmiDevice requests are used by the nnmi Integration Service.
 * <p>
 * Log the request and return an APXML message that contains the nnmi result.
 */
function handleRequest(apxml)
{
  log.debug( "handleRequest received apxml: " + apxml ); 

  var responseApXml = initializeResponse(apxml);

  try
  {
    var response = handleResponse(apxml);
    responseApXml.setToken("apiaResult", response, APXMLToken.Type.STRING);
  }
  catch (e)
  {
    log.error("Caught exception processing request. Exception: " + e)
    responseApXml.setToken("apiaResultException", e.toString(), APXMLToken.Type.STRING);
  }
  return responseApXml;  
}

/**
 * Call appropriate NNMI Webservices based on user response from xMatters notification
 */  
function handleResponse(apxml)
{
  var requestText = apxml.getValue("request_text");
  var messageText = apxml.getValue("message_text");
  var incidentId = apxml.getValue("incident_id");
  var nnmid; 
  var newPriority;
  var owner;
  
  log.debug("incidentId: " + incidentId);
  log.debug("requestText: " + requestText);
  log.debug("messageText: " + messageText);
  
  // deserialize the additional tokens
  if (apxml.getValue("additionalTokens") != null && apxml.getValue("additionalTokens") != "") 
  {
    var additionalTokens = xStream.fromXML(apxml.getValue("additionalTokens")); // LinkedHashMap
    
    log.debug( "additionalTokens: " + additionalTokens);
    nnmid = additionalTokens.get("nnmid"); 
    newPriority = additionalTokens.get("newPriority");
	owner = additionalTokens.get("owner");
  }
  else
  {
    var errorMessage = "additionalTokens is missing from apxml";
    log.debug(errorMessage);
    return errorMessage; 
  }
  
  if (!incidentId && !nnmid)
  {
     var errorMessage = "Request Script Failed: Request not associated with an Incident";
     log.error(errorMessage);
     return errorMessage;  
  }
  
  log.info("Processing " + requestText + " type response");
  
  if (requestText == "acknowledge")
  {
      log.debug(requestText);
	  if( !nnmid )
	  {
	    nnmid = nnmiResponse.getNNMiIncidentId( incidentId );
	  }
	  nnmiResponse.acknowledge(nnmid, owner);
	  
	  if(messageText)
	  {
	    nnmiResponse.annotate(nnmid, messageText);
	  }
  }
  else if (requestText == "close")
  {
      log.debug(requestText);
  	  if( !nnmid )
	  {
	    nnmid = nnmiResponse.getNNMiIncidentId( incidentId );
	  }
	  nnmiResponse.close(nnmid);
	  
	  if(messageText)
	  {
	    nnmiResponse.annotate(nnmid, messageText);
	  }
  }
  else if (requestText == "change priority")
  {
      log.debug(requestText);
  	  if( !nnmid )
	  {
	    nnmid = nnmiResponse.getNNMiIncidentId( incidentId );
	  }
	  nnmiResponse.changePriority(nnmid, newPriority);
	  
	  if(messageText)
	  {
	    nnmiResponse.annotate(nnmid, messageText);
	  }
  }
  else
  {
      var errorMessage = "Unexpected request_text value: " + requestText; 
      log.error(errorMessage);
      return errorMessage;
  }
}

/**
 * Initializes the response to be returned to the device
 * @param apxml APXML containing the request information
 * @return initialized instance of the response APXML
 */
function initializeResponse(apxml)
{
  var response = ServiceAPI.createAPXML();
  response.setMethod(APXMLMessage.RESPONSE_METHOD);
  response.setSubclass(APXMLMessage.RESPONSE_SUBCLASS);
  
  response.setToken(APXMLMessage.REQUEST_ID, apxml.getValue(APXMLMessage.REQUEST_ID), APXMLToken.Type.STRING);
  response.setToken(APXMLMessage.DESTINATION, apxml.getValue(APXMLMessage.ORIGINATOR), APXMLToken.Type.STRING);

  if (apxml.hasToken(APXMLMessage.APIA_PRIORITY)) 
  {
    response.setToken(APXMLMessage.APIA_PRIORITY, apxml.getValue(APXMLMessage.APIA_PRIORITY));
  }
  else
  {
    response.setToken(APXMLMessage.APIA_PRIORITY, APXMLMessage.APIA_PRIORITY_HIGH);
  }
  return response;
}

function getNodes(params)
{ 
  var nodeList = new ArrayList();
  var response = nnmiResponse.getNodes();
  for each( var name in response.*::getNodesResponse['return'].item.name)
  {	  
    nodeList.add(name.toString());
  }
  return nodeList;
}

